
 
const XPCOM_CONTRACTID 	= '@scenari.com/scSessionsMgr;1';
const XPCOM_CID 		= Components.ID('{A7A3079A-7FD9-4E6F-A278-45A02C9C3333}');
const XPCOM_IID 		= Components.interfaces.scISessionsMgr;

const nsICollection 		= Components.interfaces.nsICollection;
const nsIConsoleService 	= Components.interfaces.nsIConsoleService;
const nsISupportsCString 	= Components.interfaces.nsISupportsCString;
const nsISupportsString 	= Components.interfaces.nsISupportsString;
const nsIPrefService 		= Components.interfaces.nsIPrefService;
const nsIFile 				= Components.interfaces.nsIFile;
const nsIDOMNode			= Components.interfaces.nsIDOMNode;
const scISession 			= Components.interfaces.scISession;

const SUPPORTSARRAY	= '@mozilla.org/supports-array;1';
const SESSION 		= '@scenari.com/scSession;1';

const cstManifestFileName	=	"imsmanifest.xml";
const cstStorageRepName		=	"scPlayerData";


function scSessionsMgr() {
	var vFolderService = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties);
	this.fStoragePath = "";
	this.fSessionList = new Object();
	this.fDefaultStorageFile = vFolderService.get("ProfD", Components.interfaces.nsIFile);
	this.fDefaultStorageFile.append(cstStorageRepName);
}


scSessionsMgr.prototype = {
  /**
	* Initialise le workspace
	*  @param pStoragePath optionnel
	**/
	init : function(pStoragePath) {
		try{
			if(!pStoragePath){
				this.changeStoragePath(null);
			}else{
				this.changeStoragePath(pStoragePath);
			}
		}catch(e){
			debug("scSessionsMgr > init :: " + e);
		}
	},
   /**
	 * Retourne le lieu de stockage des productions
	 **/
	getStoragePath : function() {
		try{
			return this.fStoragePath;
		}catch(e){
			debug("scSessionsMgr > setStroragePath :: " + e);
		}
	},
	/**
	 *  evalue la possibilite de changer le lieu de stockage des productions 
	 *	@param : null=> path par defaut
	 **/
	canChangeStoragePath : function(pPath) {
		try{
			var vPath = pPath ? pPath : this.fDefaultStorageFile.path;
			var vDstFile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
			vDstFile.initWithPath(vPath);
			if(this.fStoragePath){
				var vSrcFile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
				vSrcFile.initWithPath(this.fStoragePath);
				if(!vSrcFile.equals(vDstFile)){// source != destination


					if(vSrcFile.exists() && !vSrcFile.isDirectory()) return Components.interfaces.scISessionsMgr.SC_ERR_ALL_DATA_NO_CONSERVED;
					if(vDstFile.exists() && !vDstFile.isDirectory()) return Components.interfaces.scISessionsMgr.SC_ERR_FILE_OUT_NO_REPOSITORY;
					if(vDstFile.exists() && vDstFile.isDirectory() && vDstFile.directoryEntries.hasMoreElements()) return Components.interfaces.scISessionsMgr.SC_ERR_FILE_OUT_NO_EMPTY;
					if(vDstFile.exists() && !vDstFile.isWritable()) return Components.interfaces.scISessionsMgr.SC_ERR_NO_WRITABLE;
					if(vDstFile.exists() && !vDstFile.diskSpaceAvailable > vDstFile.fileSize) return Components.interfaces.scISessionsMgr.SC_ERR_NOT_ENOUGH_SPACE;
				}
			}
			return 0;
		}catch(e){
			debug("scSessionsMgr > canChangeStoragePath : " + e);
			return Components.interfaces.scISessionsMgr.SC_ERR_NOT_DEFINED;
		}
	},
	/**
	 * change le lieu de stockage des productions 
	 *	@param : null=> path par defaut
	 *	@result : 0:OK, >0:echec
	 **/
	changeStoragePath : function(pPath) {
		try{
			var vPath = pPath ? pPath : this.fDefaultStorageFile.path;
			var vDstFile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
			vDstFile.initWithPath(vPath);
			if(this.canChangeStoragePath(vPath) && !vDstFile.exists()) vDstFile.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0755);// si le rpertoire de destination n'existe pas, on le cre
			var vResultCanChangeStorage = this.canChangeStoragePath(vPath);
			if(vResultCanChangeStorage==0){
				var vError = 0;
				if(this.fStoragePath){
					var vSrcFile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
					vSrcFile.initWithPath(this.fStoragePath);
					if(!vSrcFile.equals(vDstFile)){// si src=dest => pas de copie
						if(vSrcFile.exists()){
							var vSubFolderList = vSrcFile.directoryEntries;
							try{
								while(vSubFolderList.hasMoreElements()){
									var vFile = vSubFolderList.getNext().QueryInterface(nsIFile);
									vFile.copyTo(vDstFile, null);
								}
								vSrcFile.remove(true);//on ne supprime que si toute la copie s'est bien passe
							}catch(e){
								vError = Components.interfaces.scISessionsMgr.SC_ERR_NOT_DEFINED;
							}
						}
					}
				}
				for(var vIdSession in this.fSessionList){
					this.fSessionList[vIdSession].session.setBaseStoragePath(vPath);
				}
				this.fStoragePath = vPath;
				return (vError==0) ? 0 : Components.interfaces.scISessionsMgr.SC_ERR_ALL_DATA_NO_CONSERVED;

			}else if(vResultCanChangeStorage==Components.interfaces.scISessionsMgr.SC_ERR_SAME_REPOSITORY){// pas de changement necessaire : repertoire source = repertoire destination
				return 0;
			}else{return vResultCanChangeStorage;}//Echec
		}catch(e){
			debug("scSessionMgr > changeStoragePath :: " + e);
			return Components.interfaces.scISessionsMgr.SC_ERR_NOT_DEFINED;
		}
	},
   /**
	 * Retourne la liste des sessions
	 **/
	getSessionList : function() {
		try{
			var vSessionCollection = Components.classes[SUPPORTSARRAY].createInstance(nsICollection);
			for(var vIdSession in this.fSessionList){
				vSessionCollection.AppendElement(this.fSessionList[vIdSession].session);
			}
			return vSessionCollection;
		}catch(e){
			debug("scSessionsMgr > getSessionList :: " + e);
		}
	},
	
	canOpenSession : function(pUrl, pSessionUid) {
		try{
			var vStorageId 	= this.xGetId(pUrl, pSessionUid);
			var vSessionMainFile = this.xGetLocalFile(pUrl);
			if(vStorageId in this.fSessionList){
				return Components.interfaces.scISessionsMgr.SC_ERR_SESSION_ALREADY_OPEN;
			}else if(!vSessionMainFile.exists()){
				return Components.interfaces.scISessionsMgr.SC_ERR_MODULE_NO_DEFINED; // pas de fichier de description du module
			}else if(!this.xGetManifest(pUrl)){
				return Components.interfaces.scISessionsMgr.SC_ERR_MODULE_NO_MANIFEST; // pas de fichier de description du module
			}else if(!this.xGetMainUrl(pUrl)){
				return Components.interfaces.scISessionsMgr.SC_ERR_MODULE_NO_MAIN_FILE; // pas de fichier principal de l'application reconnu dans manifest
			}else{
				return 0; // OK
			}
		}catch(e){
			debug("scSessionsMgr > canOpenSession :: " + e);
			return Components.interfaces.scISessionsMgr.SC_ERR_MODULE_NO_DEFINED;
		}
	},
   /**
	 * ouvre/cree une session
	 *		pSessionUid : optionnel
	 *		retourne null si echec
	 **/
	openSession : function(pUrl, pSessionUid) {
		try{
			var vStorageId 	= this.xGetId(pUrl, pSessionUid);
			var vUrl 		= this.xGetMainUrl(pUrl);
			var vTitle 		= this.xGetTitle(pUrl) ? this.xGetTitle(pUrl) : vStorageId;
			if(this.canOpenSession(pUrl, pSessionUid)==0){
				var vSession 	= Components.classes[SESSION].createInstance(scISession);
				vSession.init(vStorageId, vUrl, this.getStoragePath(), vTitle);
				this.fSessionList[vStorageId] = {};
				this.fSessionList[vStorageId].session = vSession;
				this.fSessionList[vStorageId].storageId = vStorageId;
				return this.fSessionList[vStorageId].session;
			}else{
				return null;
			}
		}catch(e){
			debug("scSessionsMgr > openSession :: " + e);
		}
	},
   /**
	 * Supprime une session de la lise des sessions declarees
	 *	 boolean pDeleteProductions : supprimer aussi les productions
	 **/
	removeSession : function(pSessionUid, pDeleteProductions) {
		try{
			var vErrorKey = 0;
			if(pDeleteProductions) vErrorKey = this.fSessionList[pSessionUid].session.remove();
			delete this.fSessionList[pSessionUid];
			return (vErrorKey==0) ? 0 : Components.interfaces.scISessionsMgr.SC_ERR_NOT_DEFINED;
		}catch(e){
			debug("scSessionsMgr > removeSession :: " + e);
			return Components.interfaces.scISessionsMgr.SC_ERR_NOT_DEFINED;
		}
	},
	 /**
	 * Retourne  un ID unique
	 **/
	xGetId : function(pUrl, pSessionUid) {
		try{
			if(pSessionUid){
				return pSessionUid;
			}else{
				var vManifestDom = this.xGetManifest(pUrl);
				if(vManifestDom){
					var vTagOrganization = vManifestDom.getElementsByTagName("organization");
					if(vTagOrganization[0].hasAttribute("identifier")){
						return vTagOrganization[0].getAttribute("identifier");
					}else{return 0}
				}else{return 0}
			}
		}catch(e){
			debug("scSessionsMgr > xGetId :: " + e);
			return 0;
		}
	},
   /**
	 * Retourne l'URL du fichier principale du module
	 **/
	xGetMainUrl : function(pUrl) {
		try{
			var vBaseUrl = pUrl;
			var vLocalFile = this.xGetLocalFile(pUrl);
			if(!vLocalFile.isDirectory()) vBaseUrl = pUrl.substring(0,pUrl.length-vLocalFile.leafName.length-1);
			var vManifestDom = this.xGetManifest(pUrl);
			if(vManifestDom){
				var vTagOrganization = vManifestDom.getElementsByTagName("resource");
				if(vTagOrganization[0].hasAttribute("href")){
					if(vBaseUrl.indexOf("file:")!=0) vBaseUrl="file://"+vBaseUrl;
					return vBaseUrl + "/" + vTagOrganization[0].getAttribute("href");
				}else{return null}
			}else{
				return null;
			}
		}catch(e){
			debug("scSessionsMgr > xGetMainUrl :: " + e);
			return null;
		}
	},
	/**
	 * Retourne le titre du module
	 **/
	xGetTitle : function(pUrl) {
		try{
			var vManifestDom = this.xGetManifest(pUrl);
			if(vManifestDom){
				var vTitle = vManifestDom.getElementsByTagName("organization")[0].getElementsByTagName("title")[0].textContent;
				return vTitle;
			}else{
				return null;
			}
		}catch(e){
			debug("scSessionsMgr > xGetTitle :: " + e);
			return null;
		}
	},
	 /**
	 * Ouvre un fichier manifest
	 * pUrl : repertoire parent ou fichier lui meme
	 **/
	xfManifestList : new Array(),
	xGetManifest : function(pUrl){
		try{
			if(!(pUrl in this.xfManifestList)){// on conserve une trace de smanifests ouverts
				var vManifestFile = this.xGetLocalFile(pUrl);
				if(vManifestFile.exists()){
					if(vManifestFile.isDirectory()){
						vManifestFile.append("cstManifestFileName");
					}
					if(vManifestFile.exists()){
						var vIoService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
						var vChannel = vIoService.newChannel("file://"+vManifestFile.path, null, null);
						var vInput = vChannel.open();
						try {
							var vBuf = Components.classes["@mozilla.org/network/buffered-input-stream;1"].createInstance(Components.interfaces.nsIBufferedInputStream);
							vBuf.init(vInput, vChannel.contentLength);
							var vParser = Components.classes["@mozilla.org/xmlextras/domparser;1"].createInstance(Components.interfaces.nsIDOMParser);
							var vManifestDoc = vParser.parseFromStream(vBuf, null, vChannel.contentLength, "text/xml");
							var vManifestDom = vManifestDoc;
							if(vManifestDom.tagName!="parserError" && vManifestDom.namespaceURI!="http://www.mozilla.org/newlayout/xml/parsererror.xml"){
								this.xfManifestList[pUrl] = vManifestDom;
								return vManifestDom;
							}else{// fichier manifest non XML
								return null;
							}
						} finally {
							vInput.close();
						}
					}else{return null;};// pas de fichier manifest
				}else{// fichier/rpertoire inconnus
					return null;
				}
			}else{
				
				return this.xfManifestList[pUrl];
			}
		}catch(e){
			debug("scSessionsMgr > xGetManifest :: " + e);
			return null;
		}
	},
	 /**
	 * Retourne un objet nsiLocalFile selon une URL donn?e (chrome, file, ...)
	 **/
	xGetLocalFile : function(pUrl) {
		var vLocalFile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
		var runtimeSvc =  Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULRuntime);
		try{
			if(pUrl.indexOf("chrome:")==0){//chrome : rsolution pralable de l'URL pour pouvoir exploiter le fichier
				var ios =  Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
				var ch_ec = ios.newURI (pUrl, null, null);
				var chs =   Components.classes["@mozilla.org/chrome/chrome-registry;1"].getService(Components.interfaces.nsIChromeRegistry);
				var ch = chs.convertChromeURL(ch_ec);

				var vPath = runtimeSvc.OS.substring(0,3).toLowerCase() == "win" ? ch.path.replace(/\//g, '\\\\').substring(2) : ch.path;
				vLocalFile.initWithPath(unescape(vPath));
			}else{//file
				var vPath = runtimeSvc.OS.substring(0,3).toLowerCase() == "win" ? pUrl.replace(/\//g, '\\\\') : pUrl;
				vLocalFile.initWithPath(vPath);
			}
			vLocalFile.normalize();
			return vLocalFile;
		}catch(e){
			debug("scSessionsMgr > xGetLocalFile :: " + e);
			return null;
		}
	},
	 QueryInterface: function(pIId) {
        if (!pIId.equals(nsISupports) &&
            !pIId.equals(XPCOM_CID))
            throw Components.results.NS_ERROR_NO_INTERFACE;
        return this;
    }
}



function debug(pText){
	var vConsoleService = Components.classes["@mozilla.org/consoleservice;1"].getService(nsIConsoleService);
	vConsoleService.logStringMessage(pText);
}

function listProperties (pObj){
	var vRes = "";
	var vProp = null;
	for (vProp in pObj) {
		try {
			vRes += vProp +'(' + pObj[vProp]+ ')'+ "\n";
		} catch(e){
			vRes += vProp +' -> toString in error\n';
		}
	}
	return vRes;
}



var scModule = {
    registerSelf: function(compMgr, fileSpec, location, type) {
        compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.registerFactoryLocation(XPCOM_CID, 
                                        "scSessionsMgr JS component", 
                                        XPCOM_CONTRACTID, 
                                        fileSpec, 
                                        location,
                                        type);                            	
    },

    getClassObject: function(compMgr, cid, iid) {
        if (!cid.equals(XPCOM_CID))
            throw Components.results.NS_ERROR_NO_INTERFACE;

        if (!iid.equals(Components.interfaces.nsIFactory))
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

        return scSessionsMgrFactory;
    },
    
    unregisterSelf : function mod_unreg(compMgr, location, type) {

  	},

    canUnload: function(compMgr) { return true; }
};


var scSessionsMgrFactory = {
    createInstance: function(outer, iid) {
	   	try{
	        if (outer != null)
	            throw Components.results.NS_ERROR_NO_AGGREGATION;
	        if (!iid.equals(XPCOM_IID) &&
	            !iid.equals(Components.interfaces.nsISupports))
	            throw Components.results.NS_ERROR_INVALID_ARG;
	      }catch(e){
	      	debug("scIPlayer > createInstance :: " + e);
	      }finally{
	      	return new scSessionsMgr();
	      }
    }
}


function NSGetModule(comMgr, fileSpec) { return scModule; }

